# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long

from knack.arguments import CLIArgumentType


# pylint: disable=too-many-statements
def load_arguments(self, _):
    from azure.cli.core.commands.parameters import (
        resource_group_name_type,
        get_resource_name_completion_list,
        get_location_type,
        get_enum_type,
        tags_type)

    from azure.mgmt.deploymentmanager.models import (
        DeploymentMode)

    service_topology_name_type = CLIArgumentType(options_list='--service-topology-name', metavar='NAME', completer=get_resource_name_completion_list('Microsoft.DeploymentManager/servicetopologies'))
    service_name_type = CLIArgumentType(options_list='--service-name', metavar='NAME')
    artifact_source_type = CLIArgumentType(options_list='--artifact-source', metavar='NAME')

    with self.argument_context('deploymentmanager artifact-source') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx))
        c.argument('artifact_source_name', options_list=['--artifact-source-name', '--name', '-n'], help='The name of the artifact source')
        c.argument('sas_uri', options_list=['--sas-uri'], help='The SAS Uri to the Azure Storage container where the artifacts are stored.')
        c.argument('artifact_root', options_list=['--artifact-root'], help='The root folder under which the artifacts are to be found. This is the path relative to the Azure storage container provided in --sas-uri.')
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager artifact-source create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx), required=True)
        c.argument('artifact_source_name', options_list=['--artifact-source-name', '--name', '-n'], help='The name of the artifact source')
        c.argument('sas_uri', options_list=['--sas-uri'], help='The SAS Uri to the Azure Storage container where the artifacts are stored.')
        c.argument('artifact_root', options_list=['--artifact-root'], help='The root folder under which the artifacts are to be found. This is the path relative to the Azure storage container provided in --sas-uri.')
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager artifact-source update') as c:
        c.argument('sas_uri', options_list=['--sas-uri'], help='The SAS Uri to the Azure Storage container where the artifacts are stored.')
        c.argument('artifact_root', options_list=['--artifact-root'], help='The root folder under which the artifacts are to be found. This is the path relative to the Azure storage container provided in --sas-uri.')
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service-topology') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx), required=True)
        c.argument('service_topology_name', options_list=['--service-topology-name', '--name', '-n'], help='The name of the service topology')
        c.argument('artifact_source', artifact_source_type, help='The name or resource identifier of the artifact source.')
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service-topology create') as c:
        c.argument('artifact_source', artifact_source_type, help='The name or resource identifier of the artifact source.', required=False)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service-topology update') as c:
        c.argument('artifact_source', artifact_source_type, help='The name or resource identifier of the artifact source.', required=False)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx), required=True)
        c.argument('service_topology_name', service_topology_name_type, help='The name of the service topology')
        c.argument('service_name', options_list=['--service-name', '--name', '-n'], help='The name of the service')
        c.argument('target_location', options_list='--target-location', help='The location where the resources in the service should be deployed to.')
        c.argument('target_subscription_id', options_list='--target-subscription-id', help='The Id of subscription to which the resources in the service should be deployed to.')
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service create') as c:
        c.argument('service_topology_name', service_topology_name_type, help='The name of the service topology')
        c.argument('service_name', options_list=['--service-name', '--name', '-n'], help='The name of the service')
        c.argument('target_location', options_list='--target-location', help='The location where the resources in the service should be deployed to.')
        c.argument('target_subscription_id', options_list='--target-subscription-id', help='The subscription to which the resources in the service should be deployed to.')
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service update') as c:
        c.argument('target_location', options_list='--target-location', help='The location where the resources in the service should be deployed to.')
        c.argument('target_subscription_id', options_list='--target-subscription-id', help='The subscription to which the resources in the service should be deployed to.')
        c.argument('tags', tags_type)

    target_resource_type = CLIArgumentType(options_list='--target-resource-group', help='The resource group where the resources in the service unit should be deployed to')
    deployment_mode_type = CLIArgumentType(options_list='--deployment-mode', arg_type=get_enum_type(DeploymentMode), default=DeploymentMode.incremental, help='The type of depoyment mode to be used when deploying the service unit. Possible values: Incremental, Complete')
    template_path_type = CLIArgumentType(options_list='--template-path', help='The path to the ARM template file. Either the full SAS Uri or the relative path in the artifact source for this topology.')
    parameters_path_type = CLIArgumentType(options_list='--parameters-path', help='The path to the ARM parameters file. Either the full SAS Uri or the relative path in the artifact source for this topology.')

    with self.argument_context('deploymentmanager service-unit') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx))
        c.argument('service_topology_name', service_topology_name_type, help='The name of the service topology')
        c.argument('service_name', service_name_type, help='The name of the service')
        c.argument('service_unit_name', options_list=['--service-unit-name', '--name', '-n'], help='The name of the service unit')
        c.argument('target_resource_group', target_resource_type)
        c.argument('deployment_mode', deployment_mode_type)
        c.argument('template_path', template_path_type)
        c.argument('parameters_path', parameters_path_type)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service-unit create', arg_group='Named') as c:
        c.argument('service_topology_name', service_topology_name_type, help='The name of the service topology')
        c.argument('service_name', service_name_type, help='The name of the service')
        c.argument('service_unit_name', options_list=['--service-unit-name', '--name', '-n'], help='The name of the service unit')
        c.argument('target_resource_group', target_resource_type)
        c.argument('deployment_mode', deployment_mode_type)
        c.argument('template_uri', template_path_type)
        c.argument('parameters_uri', parameters_path_type)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager service-unit update') as c:
        c.argument('service_topology_name', service_topology_name_type, help='The name of the service topology')
        c.argument('service_name', service_name_type, help='The name of the service')
        c.argument('service_unit_name', options_list=['--service-unit-name', '--name', '-n'], help='The name of the service unit')
        c.argument('target_resource_group', target_resource_type)
        c.argument('deployment_mode', deployment_mode_type)
        c.argument('template_uri', template_path_type)
        c.argument('parameters_uri', parameters_path_type)
        c.argument('tags', tags_type)

    duration_type = CLIArgumentType(options_list='--duration', help='The duration of the wait step in ISO 8601 format.')
    step_name_type = CLIArgumentType(options_list=['--step-name', '--name', '-n'], help='The name of the step', completer=get_resource_name_completion_list('Microsoft.DeploymentManager/steps'))
    step_type = CLIArgumentType(options_list='--step', help='The step object, specify either the path to a json file or provide a json string that forms the step resource. The json is expected to be of the same format as the output of the relevant `az deploymentmanager step show` command')

    with self.argument_context('deploymentmanager step') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx))
        c.argument('step_name', step_name_type)
        c.argument('duration', duration_type)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager step create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('location', get_location_type(self.cli_ctx))
        c.argument('step_name', step_name_type)
        c.argument('step', step_type)
        c.argument('duration', duration_type)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager step update') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('step_name', step_name_type)
        c.argument('step', step_type)
        c.argument('duration', duration_type)
        c.argument('tags', tags_type)

    with self.argument_context('deploymentmanager step show') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('step_name', step_name_type)

    with self.argument_context('deploymentmanager step delete') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('step_name', step_name_type)

    rollout_name_type = CLIArgumentType(options_list=['--rollout-name', '--name', '-n'], help='The name of the rollout', completer=get_resource_name_completion_list('Microsoft.DeploymentManager/rollouts'))

    with self.argument_context('deploymentmanager rollout show') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('rollout_name', rollout_name_type)

    with self.argument_context('deploymentmanager rollout delete') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('rollout_name', rollout_name_type)

    with self.argument_context('deploymentmanager rollout stop') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('rollout_name', rollout_name_type)

    with self.argument_context('deploymentmanager rollout restart') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('rollout_name', rollout_name_type)
        c.argument('skip_succeeded', options_list="--skip-succeeded", help="Skips all the steps that have succeeded in the previous retries of this rollout.")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""
Help functions to extract storage account, container name, blob name and other information from
container and/or blob URL.
"""


# pylint: disable=too-few-public-methods, too-many-instance-attributes
class StorageResourceIdentifier:
    def __init__(self, cloud, moniker):
        self.is_valid = False
        self.account_name = None
        self.container = None
        self.blob = None
        self.share = None
        self.directory = None
        self.filename = None
        self.snapshot = None
        self.sas_token = None

        from urllib.parse import urlparse
        url = urlparse(moniker)

        self._is_url = (url.scheme == 'http' or url.scheme == 'https')
        if not self._is_url:
            return

        if url.path is None:
            return

        self.account_name, type_name = url.netloc[:0 - len(cloud.suffixes.storage_endpoint) - 1]\
            .split('.', 2)

        if type_name == 'blob':
            self.container, self.blob = self._separate_path_l(url.path)
        elif type_name == 'file':
            self.share, file_path = self._separate_path_l(url.path)
            self.directory, self.filename = self._separate_path_r(file_path)
            if self.filename is None:
                self.filename = self.directory
                self.directory = ''
        else:
            self.account_name = None

        if 'snapshot=' not in url.query:
            self.sas_token = url.query
        else:
            parts = url.query.split('&')
            for p in parts:
                if p.startswith('snapshot='):
                    parts.remove(p)
                    self.snapshot = p[9:]
                self.sas_token = '&'.join(parts)

    @classmethod
    def _separate_path_l(cls, path):
        return cls._separate_path(path, lambda p: p.find('/'))

    @classmethod
    def _separate_path_r(cls, path):
        return cls._separate_path(path, lambda p: p.rfind('/'))

    @classmethod
    def _separate_path(cls, path, find_method):
        if path is None:
            return None, None

        path = path.lstrip('/')
        if not path:
            return None, None

        idx = find_method(path)
        if idx == -1:
            return path, None

        return path[:idx], path[idx + 1:]

    def is_url(self):
        return self._is_url



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from enum import Enum

# consts for nodepool related commands
# priority
CONST_SCALE_SET_PRIORITY_REGULAR = "Regular"
CONST_SCALE_SET_PRIORITY_SPOT = "Spot"

# eviction policy
CONST_SPOT_EVICTION_POLICY_DELETE = "Delete"
CONST_SPOT_EVICTION_POLICY_DEALLOCATE = "Deallocate"

# node os disk type
CONST_OS_DISK_TYPE_MANAGED = "Managed"
CONST_OS_DISK_TYPE_EPHEMERAL = "Ephemeral"

# mode
CONST_NODEPOOL_MODE_SYSTEM = "System"
CONST_NODEPOOL_MODE_USER = "User"

# consts for cluster related commands
# outbound type
CONST_OUTBOUND_TYPE_LOAD_BALANCER = "loadBalancer"
CONST_OUTBOUND_TYPE_USER_DEFINED_ROUTING = "userDefinedRouting"

# private dns zone mode
CONST_PRIVATE_DNS_ZONE_SYSTEM = "system"
CONST_PRIVATE_DNS_ZONE_NONE = "none"

# used to set identity profile (for kubelet)
CONST_MANAGED_IDENTITY_OPERATOR_ROLE = 'Managed Identity Operator'
CONST_MANAGED_IDENTITY_OPERATOR_ROLE_ID = 'f1a07417-d97a-45cb-824c-7a7467783830'

# upgrade channel
CONST_RAPID_UPGRADE_CHANNEL = "rapid"
CONST_STABLE_UPGRADE_CHANNEL = "stable"
CONST_PATCH_UPGRADE_CHANNEL = "patch"
CONST_NODE_IMAGE_UPGRADE_CHANNEL = "node-image"
CONST_NONE_UPGRADE_CHANNEL = "none"

# consts for addons
# http application routing
CONST_HTTP_APPLICATION_ROUTING_ADDON_NAME = "httpApplicationRouting"

# monitoring
CONST_MONITORING_ADDON_NAME = "omsagent"
CONST_MONITORING_LOG_ANALYTICS_WORKSPACE_RESOURCE_ID = "logAnalyticsWorkspaceResourceID"

# virtual node
CONST_VIRTUAL_NODE_ADDON_NAME = "aciConnector"
CONST_VIRTUAL_NODE_SUBNET_NAME = "SubnetName"

# dashboard
CONST_KUBE_DASHBOARD_ADDON_NAME = "kubeDashboard"

# azure policy
CONST_AZURE_POLICY_ADDON_NAME = "azurepolicy"

# ingress application gateway
CONST_INGRESS_APPGW_ADDON_NAME = "ingressApplicationGateway"
CONST_INGRESS_APPGW_APPLICATION_GATEWAY_NAME = "applicationGatewayName"
CONST_INGRESS_APPGW_APPLICATION_GATEWAY_ID = "applicationGatewayId"
CONST_INGRESS_APPGW_SUBNET_ID = "subnetId"
CONST_INGRESS_APPGW_SUBNET_CIDR = "subnetCIDR"
CONST_INGRESS_APPGW_WATCH_NAMESPACE = "watchNamespace"

# confcom
CONST_CONFCOM_ADDON_NAME = "ACCSGXDevicePlugin"
CONST_ACC_SGX_QUOTE_HELPER_ENABLED = "ACCSGXQuoteHelperEnabled"

# open service mesh
CONST_OPEN_SERVICE_MESH_ADDON_NAME = "openServiceMesh"

# azure keyvault secrets provider
CONST_AZURE_KEYVAULT_SECRETS_PROVIDER_ADDON_NAME = "azureKeyvaultSecretsProvider"
CONST_SECRET_ROTATION_ENABLED = "enableSecretRotation"
CONST_ROTATION_POLL_INTERVAL = "rotationPollInterval"

# all supported addons
ADDONS = {
    'http_application_routing': CONST_HTTP_APPLICATION_ROUTING_ADDON_NAME,
    'monitoring': CONST_MONITORING_ADDON_NAME,
    'virtual-node': CONST_VIRTUAL_NODE_ADDON_NAME,
    'kube-dashboard': CONST_KUBE_DASHBOARD_ADDON_NAME,
    'azure-policy': CONST_AZURE_POLICY_ADDON_NAME,
    'ingress-appgw': CONST_INGRESS_APPGW_ADDON_NAME,
    "confcom": CONST_CONFCOM_ADDON_NAME,
    'open-service-mesh': CONST_OPEN_SERVICE_MESH_ADDON_NAME,
    'azure-keyvault-secrets-provider': CONST_AZURE_KEYVAULT_SECRETS_PROVIDER_ADDON_NAME
}

# consts for check-acr command
CONST_CANIPULL_IMAGE = "mcr.microsoft.com/aks/canipull:0.0.3-alpha"


# consts for decorator pattern
# decorator mode
class DecoratorMode(Enum):
    CREATE = 1
    UPDATE = 2


# custom exception for decorator pattern, used for gracefully exit
class DecoratorEarlyExitException(Exception):
    pass



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from datetime import datetime
from azure.cli.core.azclierror import RequiredArgumentMissingError, MutuallyExclusiveArgumentError, ArgumentUsageError
# Argument types


def datetime_type(string):
    """ Validate UTC datettime in accepted format. Examples: 31-12-2017, 31-12-2017-05:30:00 """
    accepted_date_formats = ['%d-%m-%Y', '%d-%m-%Y-%H:%M:%S']
    for form in accepted_date_formats:
        try:
            return datetime.strptime(string, form)
        except ValueError:  # checks next format
            pass
    raise ValueError("Input '{}' not valid. Valid example: 31-12-2017, 31-12-2017-05:30:00".format(string))


def validate_mi_used_for_restore_disks(vault_identity, use_system_assigned_msi, identity_id):
    if (use_system_assigned_msi or identity_id) and vault_identity is None:
        raise ArgumentUsageError("Please ensure that Selected MI is enabled for the vault")
    if use_system_assigned_msi:
        if vault_identity.type is None or "systemassigned" not in vault_identity.type.lower():
            raise ArgumentUsageError("Please ensure that System MI is enabled for the vault")
    if identity_id:
        if vault_identity.type is not None and "userassigned" in vault_identity.type.lower():
            if identity_id.lower() not in (id.lower() for id in vault_identity.user_assigned_identities.keys()):
                raise ArgumentUsageError("""
                Vault does not have the specified User MI.
                Please ensure you've provided the correct --mi-user-assigned.
                """)
        else:
            raise ArgumentUsageError("Please ensure that User MI is enabled for the vault")


def validate_crr(target_rg_id, rehydration_priority):
    if target_rg_id is None:
        raise RequiredArgumentMissingError("Please provide target resource group using --target-resource-group.")

    if rehydration_priority is not None:
        raise MutuallyExclusiveArgumentError("Archive restore isn't supported for secondary region.")
